@@ -24,8 +24,6 @@ from tkinter import filedialog, messagebox, ttk
 
 import cv2
 from appdirs import user_data_dir
-
-# from cv2.data import haarcascades
 from cv2.face import EigenFaceRecognizer_create
 from pygubu.widgets import simpletooltip
 from pygubu.widgets.scrolledframe import ScrolledFrame
@@ -52,6 +50,7 @@ class InfoWidget(WidgetABC):
         self.picked_frame_labels = []
         self.picked_frames_for_recog = []
         self.picked_frame_labels_for_recog = []
+        self.recog_picked_frame_labels_borderwidth = None
         self.frame_label_margin = 10
         self.picked_frame_label_margin = self.frame_label_margin
         self.resize_width = None
@@ -70,10 +69,36 @@ class InfoWidget(WidgetABC):
         self.new_info_added_signal = (
             self.new_info_signal
         ) = NEW_INFO_SIGNAL.OTHER.value
+        self.innerframe_for_recog = frame_labels_innerframe_for_recog = None
+        self.current_clicked_label = None
+
+    def get_frame_label_margin(self):
+        return self.frame_label_margin
+
+    def set_frame_label_margin(self, margin=10):
+        self.frame_label_margin = margin
+
+    def show_click(self, x=0, y=0, w=0, h=0, width=0, height=0):
+        self.show_current_click_label(
+            x=x, y=y, w=w, h=h, width=width, height=height
+        )
+        pass
+
+    def show_current_click_label(self, x=0, y=0, w=0, h=0, width=0, height=50):
+        if self.current_clicked_label is None:
+            return
+        self.hide_current_click_label()
+        w = width if w == 0 else w
+        h = height if h == 0 else h
+        self.current_clicked_label.place(x=x, y=y, width=w, height=h)
+        pass
+
+    def hide_current_click_label(self):
+        self.current_clicked_label.place_forget()
+        pass
 
     def del_info_id(self):
         self.fw.del_info_id()
-        self.info_id = None
 
     def get_picked_frame_labels_for_recog_len(self):
         return len(self.picked_frame_labels_for_recog)
@@ -84,7 +109,8 @@ class InfoWidget(WidgetABC):
 
     def del_picked_frame_labels_for_recog(self, update_widgets=True):
         if update_widgets:
-            for w in self.picked_frame_labels_for_recog:
+            for w in self.get_picked_frame_labels_for_recog():
+                w.place_forget()
                 w.destroy()
         self.picked_frame_labels_for_recog = []
 
@@ -94,36 +120,107 @@ class InfoWidget(WidgetABC):
     def update_picked_frame_labels_for_recog(self):
         self.update_frame_labels_for_recog()
 
-    def update_frame_labels_for_recog(self, frames=None):
-        frames = frames or self.get_picked_frames_from_frame()
-        index = 0
-        if frames is not None:
-            for f in frames:
-                label = ttk.Label(self.pick_scrolledframe_innerframe)
-                self.set_label_image(f, label)
-                label.bind(
-                    "<Button-1>",
-                    lambda event, index=index: self.recog_frame_by_index(
-                        index
-                    ),
-                )
-                label.pack(
-                    side="left",
-                    anchor="nw",
-                    padx=self.picked_frame_label_margin / 2,
-                )
-                simpletooltip.create(label, _("Click to recognize."))
-                self.add_picked_frame_label_for_recog(label)
-                index += 1
-            self.set_picked_frames_without_update_widgets(frames)
+    def set_frame_labels_innerframe_for_recog(self, innerframe=None):
+        if innerframe is None:
+            innerframe = self.pick_scrolledframe_innerframe
+        self.frame_labels_innerframe_for_recog = (
+            self.innerframe_for_recog
+        ) = innerframe
+        pass
 
+    def set_frame_labels_innerframe_forrecog(self, innerframe=None):
+        if innerframe is None:
+            innerframe = self.pick_scrolledframe_innerframe
+        self.set_frame_labels_image_use_picked_frames_for_recog(innerframe)
+        pass
+
+    def get_frame_labels_innerframe_forrecog(self):
+        return self.innerframe_for_recog
+
+    def get_frame_labels_innerframe_for_recog(self):
+        return self.get_frame_labels_innerframe_forrecog()
+
+    def del_recogframe_by_index(self, index=None):
+        self.del_picked_frame_for_recog_by_index(index)
+        pass
+
+    def del_picked_frame_for_recog_by_index(
+        self, index=0, update_widgets=True
+    ):
+        if self.is_test():
+            print(
+                "index",
+                index,
+                "picked_frames_for_recog len",
+                self.get_picked_frames_for_recog_len(),
+            )
+        if index < self.get_picked_frames_for_recog_len():
+            del self.picked_frames_for_recog[index]
+            if update_widgets:
+                self.update_frame_labels_for_recog()
+            pass
+        else:
+            print("Out of range.")
+        pass
+
+    def update_frame_labels_for_recog_use_video_frame(self, frames=None):
+        if self.get_picked_frames_for_recog_len() > 0:
+            frames = frames + self.picked_frames_for_recog
+        if frames is not None:
+            self.update_frame_labels_for_recog(frames)
+            self.set_picked_frames_for_recog(frames)
         else:
             self.set_msg(_("Nothing picked."))
             if self.is_test():
                 print(_("Picked frame is None."))
         pass
 
-    def del_picked_frames_for_recog(
+    def update_frame_labels_for_recog(self, frames=None):
+        if not frames:
+            frames = self.picked_frames_for_recog
+        self.clear_picked_frame_labels_for_recog()
+        if frames == None:
+            self.mk_tmsg("frames is None.")
+            return
+        index = 0
+        for f in frames:
+            label = ttk.Label(self.get_frame_labels_innerframe_forrecog())
+            self.set_label_image(f, label)
+            label.bind(
+                "<Button-1>",
+                lambda event, idx=index: self.recog_frame_by_index(idx),
+            )
+            label.bind(
+                "<Button-3>",
+                lambda event, idx=index: self.del_recogframe_by_index(idx),
+            )
+            label.pack(
+                side="left",
+                anchor="nw",
+                padx=self.picked_frame_label_margin / 2,
+            )
+            simpletooltip.create(
+                label,
+                _("Click to recognize,") + _("right click to delete."),
+            )
+            self.add_picked_frame_label_for_recog(label)
+            index += 1
+        # self.set_picked_frames_without_update_widgets(frames)
+
+    def set_picked_frames_for_recog(self, frames=None):
+        if frames is None:
+            return
+
+        self.picked_frames_for_recog = frames
+
+    def get_fw_pick_scrolledframe(self):
+        return self.get_fw_pick_scrolledframe_forrecog()
+
+    def get_fw_pick_scrolledframe_forrecog(self):
+        scrolledframe = self.fw.get_pick_scrolledframe_forrecog()
+        return scrolledframe
+
+    def del_picked_frames_for_recog_deprecated(
         self, update_widgets=True, update_label_list=True
     ):
         self.picked_frames_for_recog = []
@@ -172,6 +269,9 @@ class InfoWidget(WidgetABC):
 
     def get_saved_frames(self):
         if self.saved_frames is None:
+            info_id = self.get_info_id()
+            if info_id is None:
+                return None
             return self.get_saved_frames_by_id()
         return self.saved_frames
 
@@ -200,8 +300,7 @@ class InfoWidget(WidgetABC):
         return self.picked_frame_labels_for_recog
 
     def clear_picked_frame_labels_for_recog(self):
-        for l in self.get_picked_frame_labels_for_recog():
-            l.destroy()
+        self.del_picked_frame_labels_for_recog()
         pass
 
     def clear_all_frame_labels(self):
@@ -247,7 +346,7 @@ class InfoWidget(WidgetABC):
 
     def set_action_to_read(self):
         if not self.is_action_read():
-            self.del_picked_frames_for_recog()
+            # self.del_picked_frames_for_recog()
             self.delete_button_place()
             self.set_action(ACTION.READ)
             pass
@@ -255,7 +354,7 @@ class InfoWidget(WidgetABC):
     def set_action_to_pick(self):
         if not self.is_action_pick():
             self.del_picked_frames()
-            self.del_picked_frames_for_recog()
+            # self.del_picked_frames_for_recog()
             self.set_action(ACTION.PICK)
 
     def set_action_to_recog(self):
@@ -267,9 +366,14 @@ class InfoWidget(WidgetABC):
         return self.fw.get_action()
 
     def get_info_id(self):
-        self.info_id = self.fw.get_info_id()
+        if not self.info_id:
+            self.info_id = self.fw.get_info_id()
         return self.info_id
 
+    def del_info_id(self):
+        self.fw.del_info_id()
+        self.info_id = None
+
     def set_info_id(self, _id):
         self.fw.set_info_id(_id=_id)
         self.info_id = _id
@@ -554,9 +658,9 @@ class InfoWidget(WidgetABC):
     def set_adding_status(self):
         self.set_action_to_pick()
         self.del_info_id()
-        self.del_picked_frames_for_recog()
-        self.del_picked_frames()
+        # self.del_picked_frames_for_recog()
         self.del_saved_frames()
+        self.del_picked_frames()
         self.clear_info_widget_area_content()
 
     def update_widgets_by_info_id(self, info_id=None):
@@ -686,7 +790,7 @@ class InfoWidget(WidgetABC):
         basic_info = self.get_basic_info_by_info_id(info_id)
         deleted_paths = [basic_info_path, info_path, image_dir_path]
         for src in deleted_paths:
-            if os.path.exists(p):
+            if os.path.exists(src):
                 shutil.move(src, get_new_backup_path())
         self.set_basic_infos(to_none=True)
         self.update_saved_info_combobox_values()
@@ -705,18 +809,17 @@ class InfoWidget(WidgetABC):
         self.del_saved_info_by_info_id()
         pass
 
-    def fw_play_button_command(self):
-        self.fw.play_button_command()
-
     def add_button_command(self):
         self.set_adding_status()
-        self.fw_play_button_command()
+        self.fw.play_button_command()
         pass
 
     def set_widgets(self):
         super().set_widgets()
         self.set_frame_widget()
 
+        self.current_clicked_label = ttk.Label(self.root, background="blue")
+
         self.saved_info_combobox = ttk.Combobox(
             self.root,
             textvariable=self.saved_info_combobox_var,
@@ -762,6 +865,9 @@ class InfoWidget(WidgetABC):
             command=self.delete_button_command,
         )
 
+        fw_scrolledframe = self.get_fw_pick_scrolledframe_forrecog()
+        self.set_frame_labels_innerframe_for_recog(fw_scrolledframe.innerframe)
+
     def get_frame(self, copy=False):
         return self.fw.get_frame(copy)
 
@@ -1140,7 +1246,7 @@ class InfoWidget(WidgetABC):
         """
         Set frame label images for recognition using picked_frames
         """
-        self.update_frame_labels_for_recog(frames)
+        self.update_frame_labels_for_recog_use_video_frame(frames)
 
     def get_picked_frame_by_index(self, index=None):
         if index is None:
@@ -1161,6 +1267,16 @@ class InfoWidget(WidgetABC):
             )
         return None
 
+    def get_picked_frames_for_recog_len(self):
+        return len(self.picked_frames_for_recog)
+
+    def get_picked_frame_by_index_for_recog(self, index=None):
+        if index is None:
+            return None
+        if index < self.get_picked_frames_for_recog_len():
+            return self.picked_frames_for_recog[index]
+        return None
+
     def recog_frame_by_index(self, index=None):
         self.recog_picked_frame_by_index(index)
         pass
@@ -1178,13 +1294,15 @@ class InfoWidget(WidgetABC):
     def get_info_id_by_frame(self, frame=None):
         return self.fw.get_info_id_by_frame(frame)
 
-    def recog_picked_frame_by_index(self, index=None):
+    def recog_picked_frame_by_index(self, index=None, show_click=True):
         assert index is not None
         if index is None:
             if self.is_test():
                 print("`recog_picked_frame_by_index.index` is `None`.")
             return
-        frame = self.get_picked_frame_by_index(index)
+
+        self.mk_tmsg(f"recog_picked_frame_by_index.index --> {index}")
+        frame = self.get_picked_frame_by_index_for_recog(index)
         if frame is None:
             self.set_warning_msg(_("`Picked Frames is None.`"))
             self.mk_tmsg("`recog_picked_frame_by_index.frame` is `None`.")
@@ -1196,8 +1314,56 @@ class InfoWidget(WidgetABC):
             self.mk_tmsg("`recog_picked_frame_by_index.info_id` is `None`.")
             return
         self.update_widgets_by_info_id_for_recog(info_id)
+
+        if show_click:
+            labels = self.get_picked_frame_labels_for_recog()
+
+            for l in labels:
+                l.config(borderwidth=0, background="")
+
+            label = labels[index]
+            label.config(
+                borderwidth=self.get_recog_picked_frame_labels_borderwidth(),
+                background="blue",
+            )
+
         pass
 
+    def get_recog_picked_frame_labels(self):
+        return self.get_picked_frame_labels_for_recog()
+
+    def get_recog_picked_frame_labels_len(self):
+        return len(self.get_recog_picked_frame_labels())
+
+    def get_recog_picked_frame_label_by_index(self, index=0):
+        if self.get_recog_picked_frame_labels_len() > index:
+            return self.picked_frame_labels_for_recog[index]
+        return None
+
+    def get_default_recog_picked_frame_labels_borderwidth(self):
+        return 5
+
+    def get_recog_picked_frame_labels_borderwidth(self):
+        return (
+            self.recog_picked_frame_labels_borderwidth
+            or self.get_default_recog_picked_frame_labels_borderwidth()
+        )
+
+    def set_recog_picked_frame_labels_borderwidth(self, borderwidth=None):
+        self.recog_picked_frame_labels_borderwidth = (
+            borderwidth
+            or self.get_default_recog_picked_frame_labels_borderwidth()
+        )
+
+    def get_fw_pick_scrolledframe_forrecog_x(self):
+        return self.fw.get_pick_scrolledframe_forrecog_x()
+
+    def get_fw_pick_scrolledframe_forrecog_y(self):
+        return self.fw.get_pick_scrolledframe_forrecog_y()
+
+    def get_fw_pick_scrolledframe_forrecog_height(self):
+        return self.fw.get_pick_scrolledframe_forrecog_height()
+
     def set_frame_labels_image(
         self, show_saved_frames=True, show_picked_frames=True
     ):
